package	jp.co.sds_corp.transfer.mixer;

import java.util.List;
import java.util.Vector;

import jp.co.sds_corp.transfer.IData;
import jp.co.sds_corp.transfer.IDataKey;
import jp.co.sds_corp.transfer.IReceiveListener;
import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.transfer.ReceiverSupport;
import jp.co.sds_corp.transfer.StatusChangeEvent;
import jp.co.sds_corp.transfer.filter.IFilter;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* Mixerの基底クラスです。
*/
public abstract class AbstractMixer extends AbstractService
								implements IMixer
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	transient ReceiverSupport support = new ReceiverSupport();
	Vector<IFilter>		vecFilter = new Vector<IFilter>();
	boolean		debug = false;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		String var;
		var = config.get( "debug", "false" );
		debug = var.equals( "true" );
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
	}

	/**
	* データ受信を受け取るオブジェクトを追加します。
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public void addReceiveListener( IReceiveListener l ) {
		support.addReceiveListener( l );
	}
	/**
	* データ受信を受け取るオブジェクトを削除します。
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public void removeReceiveListener( IReceiveListener l ) {
		support.removeReceiveListener( l );
	}
	/**
	* データ受信を受け取るオブジェクトを追加します。
	* @param	key		対象のキー
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public void addReceiveListener( IDataKey key, IReceiveListener l ) {
		support.addReceiveListener( key, l );
	}
	/**
	* データ受信を受け取るオブジェクトを削除します。
	* @param	key		対象のキー
	* @param	l		データ受信を受け取るオブジェクト
	*/
	public void removeReceiveListener( IDataKey key, IReceiveListener l ){
		support.removeReceiveListener( key, l );
	}
	/**
	* データ受信イベントを通知します。
	* @param	data	受信したデータ
	*/
	public void fireReceive( IData data )
		throws Exception
	{
		ReceiveEvent e = new ReceiveEvent( this, data );
		fireReceive( e );
	}

	/**
	* データ受信イベントを通知します。
	* @param	e		データ受信イベント
	*/
	public void fireReceive( ReceiveEvent e ) throws Exception {
		if ( isDebug() ) {
			logger.debug( e.toString() );
		}
		support.fireReceive( e );
	}

	/**
	* 状態変更イベントを通知します。
	* @param	e		状態変更イベント
	*/
	public void fireStatusChange( StatusChangeEvent e ) throws Exception {
		if ( isDebug() ) {
			logger.debug( e.toString() );
		}
		support.fireStatusChange( e );
	}

	/**
	* データのフィルタを設定します。
	* @param	var		設定するフィルタ
	*/
	public void addFilter( IFilter var ) {
		vecFilter.add( var );
	}

	/**
	* データのフィルタを削除します。
	* @param	var		削除するフィルタ
	*/
	public void removeFilter( IFilter var ) {
		vecFilter.remove( var );
	}

	/**
	* データのフィルタを取得します。
	* @return	設定されているフィルタの格納されている{@link List}
	*/
	public List<IFilter> getFilterList() {
		return vecFilter;
	}

	/**
	* データのフィルタリングを行なう
	* @param	e		データ受信イベント
	*/
	public boolean isValid( ReceiveEvent e ) throws Exception {
		List<IFilter> list;
		list = getFilterList();
		for ( int i = 0; i < list.size(); i++ ) {
			IFilter flt = (IFilter )list.get( i );
			if ( flt.isTarget( e )) {
				if ( !flt.isValid( e )) {
					flt.putLog( "[MIX]", e );
					return false;
				}
			}
		}
		return true;
	}

	/**
	* デバッグフラグ(debug)を設定します。
	* @param	var		設定するデータ
	*/
	public void setDebug( boolean var ) {
		this.debug = var ;
	}
	/**
	* デバッグフラグ(debug)を取得します。
	* @return	debugの内容
	*/
	public boolean isDebug() {
		return debug;
	}

}
