package	jp.co.sds_corp.transfer.output;

import javax.jms.JMSException;

import jp.co.sds_corp.transfer.ReceiveEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.msg.IMessage;
import jp.co.sds_corp.util.msg.IMessageAdapter;
import jp.co.sds_corp.util.msg.MessageAdapterFactory;

/**
* JMSを使用してデータを外部に送信します。
設定内容の詳細については、{@link MessageAdapterFactory}のマニュアルを参照してください。
<PRE>
&lt;Output name="JMS" class="jp.co.sds_corp.transfer.output.JMSAdapter"&gt;
	&lt;Type&gt;jmsTopic&lt;/Type&gt;
	&lt;InitialContext&gt;weblogic.jndi.WLInitialContextFactory&lt;/InitialContext&gt;
	&lt;ConnectionFactory&gt;jms/topic/ConnectionFactory&lt;/ConnectionFactory&gt;
	&lt;URL&gt;http://localhost:8080&lt;/URL&gt;
	&lt;User&gt;user&lt;/User&gt;
	&lt;Password&gt;password&lt;/Password&gt;
	&lt;Send&gt;jms/topic/result&lt;/Send&gt;
&lt;/Output&gt;
</PRE>
*/
public class JMSAdapter extends AbstractOutput
{
	private static final long serialVersionUID = 1L;
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	MessageAdapterFactory factory;
	IMessageAdapter	adapter;

	/**
	* 指定された初期情報で初期化します。
	* @param	config	初期情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		try {
			factory = new MessageAdapterFactory();
			adapter = factory.connect( config );

			adapter.createSender( config.get( "Send" ));
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( "linked exception:", le );
			}
			throw( je );
		}
	}

	/**
	* このサービスを開始します。
	*/
	public void start() {
		try {
			adapter.start();
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( "linked exception:", le );
			}
			logger.error( "start error", je );
		} catch ( Exception e ) {
			logger.error( "start error", e );
		}
	}

	/**
	* このサービスを停止します。
	*/
	public void stop() {
		try {
			adapter.stop();
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( "linked exception:", le );
			}
			logger.error( "stop error", je );
		} catch ( Exception e ) {
			logger.error( "stop error", e );
		}
	}

	/**
	* メッセージアダプタを返します。
	*/
	public IMessageAdapter getAdapter() {
		return adapter;
	}

	/**
	* データに何らかの変更があったときに呼び出されます。
	* @param	e		イベントソースおよびデータの変更内容を指定します。
	*/
	public void received( ReceiveEvent e ) throws Exception {
		if ( isDebug() ) {
			logger.debug( e.toString() );
		}
		try {
			IMessage msg = adapter.createMessage( IMessage.Type.OBJECT );
			msg.setObject( "Event", e );
			adapter.writeMessage( msg, 5000 );
		} catch ( JMSException je ) {
			Exception le = je.getLinkedException();
			if ( le != null ) {
				logger.error( "linked exception:", le );
			}
			throw( je );
		}
	}
}
